@@ -1,4 +1,4 @@
-/*	$OpenBSD: ls.c,v 1.21 2003/08/06 19:09:09 tedu Exp $	*/
+/*	$OpenBSD: ls.c,v 1.27 2005/11/30 00:57:25 deraadt Exp $	*/
 /*	$NetBSD: ls.c,v 1.18 1996/07/09 09:16:29 mycroft Exp $	*/
 
 /*
@@ -43,7 +43,7 @@ static char copyright[] =
 #if 0
 static char sccsid[] = "@(#)ls.c	8.7 (Berkeley) 8/5/94";
 #else
-static char rcsid[] = "$OpenBSD: ls.c,v 1.21 2003/08/06 19:09:09 tedu Exp $";
+static char rcsid[] = "$OpenBSD: ls.c,v 1.27 2005/11/30 00:57:25 deraadt Exp $";
 #endif
 #endif /* not lint */
 
@@ -61,6 +61,7 @@ static char rcsid[] = "$OpenBSD: ls.c,v 1.21 2003/08/06 19:09:09 tedu Exp $";
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include <util.h>
 
 #include "ls.h"
 #include "extern.h"
@@ -90,7 +91,6 @@ int f_inode;			/* print inode */
 int f_listdir;			/* list actual directory, not contents */
 int f_listdot;			/* list files beginning with . */
 int f_longform;			/* long listing format */
-int f_newline;			/* if precede with newline */
 int f_nonprint;			/* show unprintables as ? */
 int f_nosort;			/* don't sort output */
 int f_numericonly;		/* don't expand uid to symbolic name */
@@ -101,10 +101,8 @@ int f_singlecol;		/* use single column output */
 int f_size;			/* list size in short listing */
 int f_statustime;		/* use time of last mode change */
 int f_stream;			/* stream format */
-int f_dirname;			/* if precede with directory name */
 int f_type;			/* add type character for non-regular files */
 int f_typedir;			/* add type character for directories */
-int f_whiteout;			/* show whiteout entries */
 
 int rval;
 
@@ -133,7 +131,7 @@ ls_main(int argc, char *argv[])
 		f_listdot = 1;
 
 	fts_options = FTS_PHYSICAL;
-	while ((ch = getopt(argc, argv, "1ACFLRSTWacdfghiklmnopqrstux")) != -1) {
+	while ((ch = getopt(argc, argv, "1ACFLRSTacdfghiklmnopqrstux")) != -1) {
 		switch (ch) {
 		/*
 		 * The -1, -C and -l, -m and -x options all override each
@@ -235,9 +233,6 @@ ls_main(int argc, char *argv[])
 		case 't':
 			sortkey = BY_TIME;
 			break;
-		case 'W':
-			f_whiteout = 1;
-			break;
 		default:
 			usage();
 		}
@@ -260,14 +255,6 @@ ls_main(int argc, char *argv[])
 	if (!f_longform && !f_listdir && !f_type)
 		fts_options |= FTS_COMFOLLOW;
 
-	/*
-	 * If -W, show whiteout entries
-	 */
-#ifdef FTS_WHITEOUT
-	if (f_whiteout)
-		fts_options |= FTS_WHITEOUT;
-#endif
-
 	/* If -l or -s, figure out block size. */
 	if (f_longform || f_size) {
 		if (!kflag)
@@ -328,7 +315,7 @@ ls_main(int argc, char *argv[])
 		traverse(argc, argv, fts_options);
 	else
 		traverse(1, dotav, fts_options);
-	exit(rval);
+	return (rval);
 }
 
 static int output;			/* If anything output. */
@@ -390,7 +377,8 @@ traverse(int argc, char *argv[], int options)
 			break;
 		case FTS_DNR:
 		case FTS_ERR:
-			warnx("%s: %s", p->fts_name, strerror(p->fts_errno));
+			warnx("%s: %s", p->fts_name[0] == '\0' ? p->fts_path :
+			    p->fts_name, strerror(p->fts_errno));
 			rval = 1;
 			break;
 		}
@@ -539,8 +527,11 @@ display(FTSENT *p, FTSENT *list)
 		d.s_inode = strlen(buf);
 		(void)snprintf(buf, sizeof(buf), "%lu", maxnlink);
 		d.s_nlink = strlen(buf);
-		(void)snprintf(buf, sizeof(buf), "%qu", maxsize);
-		d.s_size = strlen(buf);
+		if (!f_humanval) {
+			(void)snprintf(buf, sizeof(buf), "%qu", maxsize);
+			d.s_size = strlen(buf);
+		} else
+			d.s_size = FMT_SCALED_STRSIZE-2; /* no - or '\0' */
 		d.s_user = maxuser;
 	}
 
